-----------------------------------------------------------------
--  lcd.vhd
--			LCD-Interface fuer LCD124
--			=========================
--
--  07.10.97	Bodo Wenzel	Erstellung
--  26.03.98	Bodo Wenzel	Hardware fertig
--  06.05.98	Bodo Wenzel	Timing verbessert
--  25.05.98	Bodo Wenzel	Gatterzahl erniedrigt
-----------------------------------------------------------------

-- Interface ----------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity lcd is
  port (clock: in  std_ulogic;
        addr:  out std_ulogic_vector(14 downto 0);
        data:  in  std_ulogic_vector(7 downto 0);
        s:     out std_ulogic;
        cp1:   out std_ulogic;
        cp2:   out std_ulogic;
        du:    out std_ulogic_vector(3 downto 0);
        dl:    out std_ulogic_vector(3 downto 0));
end;

-- Implementation -----------------------------------------------

architecture beh of lcd is
  constant ADDR_MSB_MAX:  unsigned(7 downto 0)
                          := conv_unsigned(16000/128-1,8);
  constant ADDR_MSB_OFFS: unsigned(7 downto 0)
                          := conv_unsigned(16000/128,8);
  constant ADDR_LSB_MAX:  unsigned(6 downto 0)
                          := (others => '1');

  signal h_count: natural range 0 to 79 := 0;

  signal s_p1:  std_ulogic;
  signal s_p2:  std_ulogic;
  signal i_cp1: std_ulogic;
  signal i_cp2: std_ulogic := '0';

  signal nibble: boolean := FALSE;

  signal addr_msb1: unsigned(7 downto 0) := (others => '0');
  signal addr_msb2: unsigned(7 downto 0);
  signal addr_lsb:  unsigned(6 downto 0) := (others => '0');

  signal data_nibble: std_ulogic_vector(3 downto 0);
  signal du_latch:    std_ulogic_vector(3 downto 0);
begin
  pulses: process (clock)
  begin
    if falling_edge(clock) then
      i_cp1 <= '0';
      i_cp2 <= not i_cp2;

      if i_cp2='0' then
        nibble <= not nibble;
      end if;

      if i_cp2='1' and nibble and h_count=0 then
        i_cp1 <= '1';
        s     <= s_p2;
        s_p2  <= s_p1;
        s_p1  <= '0';
      end if;

      if (addr_msb1=ADDR_MSB_MAX and addr_lsb=ADDR_LSB_MAX) then
        s_p1  <= '1';
      end if;
    end if;
  end process;

  cp1 <= i_cp1;
  cp2 <= i_cp2;

  address: process (clock)
  begin
    if falling_edge(clock) then
      if i_cp2='0' and (not nibble) then
        if addr_lsb=ADDR_LSB_MAX then
          if addr_msb1=ADDR_MSB_MAX then
            addr_msb1 <= (others => '0');
          else
            addr_msb1 <= addr_msb1+1;
          end if;
        end if;

        addr_lsb <= addr_lsb+1;
      end if;
    end if;
  end process;

  addr_msb2 <= addr_msb1+ADDR_MSB_OFFS;
  addr(14 downto 7) <=
         std_ulogic_vector(addr_msb1) when i_cp2='1'
    else std_ulogic_vector(addr_msb2);
  addr(6 downto 0) <= std_ulogic_vector(addr_lsb);

  line_count: process (clock)
  begin
    if falling_edge(clock) then
      if i_cp2='0' and (not nibble) then
        if (addr_msb1=ADDR_MSB_MAX and addr_lsb=ADDR_LSB_MAX) or
           h_count=79 then
          h_count <= 0;
        else
          h_count <= h_count+1;
        end if;
      end if;
    end if;
  end process;

  data_nibble <= data(7 downto 4) when nibble
            else data(3 downto 0);

  latch: process (clock)
  begin
    if falling_edge(clock) then
      if i_cp2='1' then
        du_latch <= not data_nibble;
      else
        du <= du_latch;
        dl <= not data_nibble;
      end if;
    end if;
  end process;
end;

-- Ende ---------------------------------------------------------
