-----------------------------------------------------------------
--  lcd124tb.vhd
--			Test von lcd124.vhd
--			===================
--
--  26.09.97	Bodo Wenzel	Test der PLL
--  04.10.97	Bodo Wenzel	Test des LCD-Interfaces
--  07.10.97	Bodo Wenzel	Test des Video-Interfaces
--  24.03.98	Bodo Wenzel	Hardware fertig
--  28.05.98	Bodo Wenzel	Anpassungen
-----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity tb is
end;

architecture beh of tb is
  component lcd124
  port (clock:    in    std_ulogic;
        vsync_jp: inout std_logic_vector(4 downto 0);
        hfreq_jp: inout std_logic_vector(1 downto 0);
        hsync_jp: inout std_logic_vector(1 downto 0);
        pixel_jp: inout std_logic_vector(2 downto 0);
        vsync:    in    std_ulogic;
        hsync:    in    std_ulogic;
        mono:     in    std_ulogic;
        pll:      out   std_ulogic;
        rom_ram:  out   std_ulogic;
        n_oe:     out   std_ulogic;
        n_we:     out   std_ulogic;
        addr:     out   std_ulogic_vector(14 downto 0);
        data:     inout std_logic_vector(7 downto 0);
        s:        out   std_ulogic;
        cp1:      out   std_ulogic;
        cp2:      out   std_ulogic;
        du:       out   std_ulogic_vector(3 downto 0);
        dl:       out   std_ulogic_vector(3 downto 0));
  end component;

  component test
  port (set_min:  out boolean;
        set_mid:  out boolean;
        set_max:  out boolean;
        vsync_jp: out std_logic_vector(4 downto 0);
        hfreq_jp: out std_logic_vector(1 downto 0);
        hsync_jp: out std_logic_vector(1 downto 0);
        pixel_jp: out std_logic_vector(2 downto 0);
        vsync:    out std_ulogic;
        hsync:    out std_ulogic;
        mono:     out std_ulogic);
  end component;

  component osc
  port (set_min: in  boolean;
        set_mid: in  boolean;
        set_max: in  boolean;
        pll:     in  std_ulogic;
        clock:   out std_ulogic);
  end component;

  component ram
  port (addr: in    std_ulogic_vector(14 downto 0);
        data: inout std_logic_vector(7 downto 0);
        n_ce: in    std_ulogic;
        n_we: in    std_ulogic;
        n_oe: in    std_ulogic);
  end component;

  component rom
  port (addr: in  std_ulogic_vector(14 downto 0);
        data: out std_logic_vector(7 downto 0);
        n_ce: in  std_ulogic);
  end component;

  signal set_min: boolean;
  signal set_mid: boolean;
  signal set_max: boolean;

  signal clock: std_ulogic;

  signal vsync_jp: std_logic_vector(4 downto 0);
  signal hfreq_jp: std_logic_vector(1 downto 0);
  signal hsync_jp: std_logic_vector(1 downto 0);
  signal pixel_jp: std_logic_vector(2 downto 0);

  signal vsync: std_ulogic;
  signal hsync: std_ulogic;
  signal mono:  std_ulogic;

  signal pll:     std_ulogic;

  signal addr:  std_ulogic_vector(14 downto 0);
  signal data:  std_logic_vector(7 downto 0);
  signal n_ram: std_ulogic;
  signal n_rom: std_ulogic;
  signal n_we:  std_ulogic;
  signal n_oe:  std_ulogic;

  signal s:   std_ulogic;
  signal cp1: std_ulogic;
  signal cp2: std_ulogic;
  signal du:  std_ulogic_vector(3 downto 0);
  signal dl:  std_ulogic_vector(3 downto 0);
begin
  c_lcd124: lcd124 port map
    (clock,
     vsync_jp,hfreq_jp,hsync_jp,pixel_jp,
     vsync,hsync,mono,
     pll,
     n_rom,n_oe,n_we,addr,data,
     s,cp1,cp2,du,dl);

  c_test: test port map
    (set_min,set_mid,set_max,
     vsync_jp,hfreq_jp,hsync_jp,pixel_jp,
     vsync,hsync,mono);

  c_osc: osc port map
    (set_min,set_mid,set_max,pll,clock);

  c_ram: ram port map
    (addr,data,n_ram,n_we,n_oe);

  c_rom: rom port map
    (addr,data,n_rom);

  n_ram <= not n_rom;
end;

-- Signale erzeugen ---------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity test is
  port (set_min:  out boolean;
        set_mid:  out boolean;
        set_max:  out boolean;
        vsync_jp: out std_logic_vector(4 downto 0);
        hfreq_jp: out std_logic_vector(1 downto 0);
        hsync_jp: out std_logic_vector(1 downto 0);
        pixel_jp: out std_logic_vector(2 downto 0);
        vsync:    out std_ulogic;
        hsync:    out std_ulogic;
        mono:     out std_ulogic);
end;

-- PLL  - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

architecture test_pll of test is
begin
  vsync_jp <= (others => '0');
  hfreq_jp <= (others => '0');
  hsync_jp <= (others => '0');
  pixel_jp <= (others => '0');

  mono <= '1';

  process
  begin
    set_min <= TRUE, FALSE after 1 us;
    set_mid <= FALSE;
    set_max <= FALSE;
    wait for 1 ms;
    set_max <= TRUE, FALSE after 1 us;
    wait for 1 ms;
    assert FALSE report "Test beendet." severity NOTE;
    wait;
  end process;

  process
  begin
    loop
      vsync <= '1';
      hsync <= '0';
      wait for 25 us;
      hsync <= '1';
      wait for 3 us;
      vsync <= '0';
      for i in 1 to 50 loop
        hsync <= '0';
        wait for 25 us;
        hsync <= '1';
        wait for 3 us;
      end loop;
    end loop;
  end process;
end;

-- Video-Interface  - - - - - - - - - - - - - - - - - - - - - - -

architecture test_video of test is
begin
  process
  begin
    vsync_jp <= "00000";
    hfreq_jp <= "00";
    hsync_jp <= "00";
    pixel_jp <= "000";
    wait for 2 ms;
    vsync_jp <= "01010";
    hfreq_jp <= "01";
    hsync_jp <= "10";
    pixel_jp <= "100";
    wait for 2 ms;
    vsync_jp <= "10101";
    hfreq_jp <= "11";
    hsync_jp <= "11";
    pixel_jp <= "111";
    wait;
  end process;

  process
  begin
    set_min <= FALSE;
    set_mid <= TRUE;
    set_max <= FALSE;
    wait for 5490 us;
    assert FALSE report "Test beendet." severity NOTE;
    wait;
  end process;

  process
    variable pixel:   natural;
    variable pattern: std_ulogic_vector(15 downto 0);
  begin
    vsync <= '1';
    hsync <= '1';
    wait for 200 us;
    loop
      vsync <= '1';
      hsync <= '0';
      wait for 25 us;
      hsync <= '1';
      wait for 3 us;
      vsync <= '0';
      for i in 1 to 30 loop
        hsync <= '0';
        wait for 25 us;
        hsync <= '1';
        wait for 3 us;
      end loop;
      pixel := 0;
      for i in 1 to 20 loop
        hsync <= '0';
        for j in 1 to 50 loop
          pattern := std_ulogic_vector(conv_unsigned(pixel,16));
          for k in 15 downto 0 loop
            mono <= pattern(k);
            wait for 31250 ps;
          end loop;
          pixel := pixel+1;
        end loop;
        mono <= '1';
        hsync <= '1';
        wait for 3 us;
      end loop;
      for i in 1 to 10 loop
        hsync <= '0';
        wait for 25 us;
        hsync <= '1';
        wait for 3 us;
      end loop;
    end loop;
  end process;
end;

-- LCD-Interface  - - - - - - - - - - - - - - - - - - - - - - - -

architecture test_lcd of test is
begin
  vsync_jp <= (others => '0');
  hfreq_jp <= (others => '0');
  hsync_jp <= (others => '0');
  pixel_jp <= (others => '0');

  process
  begin
    mono <= '1';
    wait for 42 us;
    mono <= '0';
    wait for 23 us;
  end process;

  process
  begin
    set_min <= FALSE;
    set_mid <= FALSE;
    set_max <= FALSE;
    wait for 2490 us;
    assert FALSE report "Test beendet." severity NOTE;
    wait;
  end process;

  process
  begin
    vsync <= '1';
    hsync <= '1';
    wait for 1 ms;
    vsync <= '0';
    hsync <= '0';
    wait;
  end process;
end;

-- Zusaetze XC3000 ----------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity gclk is
  port (i: in  std_ulogic;
        o: out std_ulogic);
end;

architecture beh of gclk is
begin
  o <= i;
end;

--  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

library ieee;
use ieee.std_logic_1164.all;

entity aclk is
  port (i: in  std_ulogic;
        o: out std_ulogic);
end;

architecture beh of aclk is
begin
  o <= i;
end;

--  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

library ieee;
use ieee.std_logic_1164.all;

entity pullup is
  port (o: out std_logic);
end;

architecture beh of pullup is
begin
  o <= 'H';
end;

--  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

library ieee;
use ieee.std_logic_1164.all;

entity ibuf is
  port (i: in  std_logic;
        o: out std_ulogic);
end;

architecture beh of ibuf is
begin
  o <= i;
end;

-- Oszillator ---------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity osc is
  port (set_min: in  boolean;
        set_mid: in  boolean;
        set_max: in  boolean;
        pll:     in  std_ulogic;
        clock:   out std_ulogic);
end;

architecture beh of osc is
  constant MIN_PERIOD: time := 25 ns;
  constant MID_PERIOD: time := 31250 ps;
  constant MAX_PERIOD: time := 40 ns;

  constant FILTER_DELAY: time := 10 us;

  constant PERIOD_DIV: positive := 700;

  signal clock_period: time := MID_PERIOD;
begin
  process
  begin
    clock <= '1';
    wait for clock_period/2;
    clock <= '0';
    wait for clock_period/2;
  end process;

  process (pll,set_min,set_mid,set_max)
    variable pll_last_event: time;
    variable diff_time:      time;
  begin
    if set_min then
      clock_period <= MIN_PERIOD;
    elsif set_mid then
      clock_period <= MID_PERIOD;
    elsif set_max then
      clock_period <= MAX_PERIOD;
    elsif pll'event then
      if pll'last_value='0' and clock_period<MAX_PERIOD then
        diff_time := (now - pll_last_event)/PERIOD_DIV;
        clock_period <= clock_period + diff_time,
          clock_period + diff_time/2 after FILTER_DELAY;
      elsif pll'last_value='1' and clock_period>MIN_PERIOD then
        diff_time := (now - pll_last_event)/PERIOD_DIV;
        clock_period <= clock_period - diff_time,
          clock_period - diff_time/2 after FILTER_DELAY;
      end if;
      pll_last_event := now;
    end if;
  end process;
end;

-- Speicher -----------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity ram is
  port (addr: in    std_ulogic_vector(14 downto 0);
        data: inout std_logic_vector(7 downto 0);
        n_ce: in    std_ulogic;
        n_we: in    std_ulogic;
        n_oe: in    std_ulogic);
end;

architecture beh of ram is
  subtype byte is std_logic_vector(7 downto 0);

  type mem is array(0 to 255) of byte;

  signal m: mem;

  signal wr: std_ulogic;
  signal rd: std_ulogic;
begin
  wr <= not n_ce and not n_we;
  rd <= not n_ce and not n_oe;

  process
  begin
    for i in 0 to 255 loop
      m(i) <= std_logic_vector(conv_unsigned(i,8));
    end loop;
    wait until wr='1';
    loop
      wait until wr='0';
      m(conv_integer(unsigned(addr(7 downto 0)))) <= data;
    end loop;
  end process;

  process (rd,addr)
  begin
    if rd='1' then
      data <= m(conv_integer(unsigned(addr(7 downto 0))))
              after 100 ns;
    else
      data <= (others => 'Z') after 30 ns;
    end if;
  end process;
end;

library ieee;
use ieee.std_logic_1164.all;

entity rom is
  port (addr: in  std_ulogic_vector(14 downto 0);
        data: out std_logic_vector(7 downto 0);
        n_ce: in  std_ulogic);
end;

architecture beh of rom is
  signal ff: std_logic_vector(7 downto 0) := (others => '0');
begin
  process (n_ce,addr)
  begin
    if n_ce='0' then
      data <= (ff(6 downto 0) & ff(7)) xor
              std_logic_vector(addr(7 downto 0)) after 100 ns;
    else
      data <= (others => 'Z') after 30 ns;
    end if;
  end process;
end;

-- Ende ---------------------------------------------------------
